#! /usr/bin/env python
# -*- coding: utf-8 -*-
# pylint: disable=bad-whitespace, attribute-defined-outside-init, invalid-name
# pylint: disable=too-many-statements, wrong-import-position
"""
    New Project – A new Python project.

    This setuptools script follows the DRY principle and tries to
    minimize repetition of project metadata by loading it from other
    places (like the package's `__init__.py`). Incidently, this makes
    the script almost identical between different projects.

    It is also importable (by using the usual `if __name__ == '__main__'`
    idiom), and exposes the project's setup data in a `project` dict.
    This allows other tools to exploit the data assembling code contained
    in here, and again supports the DRY principle. The `rituals` package
    uses that to provide Invoke tasks that work for any project, based on
    its project metadata.

"""

import io
import os
import sys
import json
from pathlib import Path
from collections import defaultdict
from setuptools import setup, find_packages


# Project data (the rest is parsed from __about__.py and other project files)
name = 'new-project'
package_name = 'new_project'


# Helper functions
def project_root():
    """Returns Path object for project root (i.e. location of setup.py)"""
    path_to_setup_file = Path(__file__).resolve()
    return path_to_setup_file.parent


def srcfile(*args):
    """Takes a list of path elements and returns a Path object"""
    return project_root() / Path(*args)


def _build_metadata():
    # Add some common files to EGG-INFO
    candidate_files = [
        'LICENSE', 'NOTICE',
        'README', 'README.md', 'README.rst', 'README.txt',
        'CHANGES', 'CHANGELOG', 'debian/changelog',
    ]
    data_files = defaultdict(list)
    for filename in candidate_files:
        if srcfile(filename).exists():
            data_files['EGG-INFO'].append(filename)

    def classifiers():
        pass

    def long_description():
        project_root = Path(os.path.dirname(os.path.realpath(__file__)))
        readme = project_root / 'readme.md'
        return readme.read_text()

    def requires(requirements_file):
        requires = []
        if os.path.exists(str(srcfile(requirements_file))):
            with io.open(str(srcfile(requirements_file)), encoding='utf-8') as handle:
                for line in handle:
                    line = line.strip()
                    if line and not line.startswith('#') and ';' not in line:
                        if any(line.startswith(i) for i in ('-e', 'http://', 'https://')):
                            line = line.split('#egg=')[1]
                        requires.append(line)
        return requires

    def about(metadata):
        about_file = project_root() / 'src' / package_name / 'about.py'

        about = {}
        exec(about_file.read_text(), about)

        return about[metadata]

    def entry_points():
        console_scripts = []
        appname = about('appname')
        script = '{0} = {1}.cli.entry_point:main'.format(appname, package_name)
        console_scripts.append(script)
        return {'console_scripts': console_scripts}

    metadata = {
        'name'                : package_name,
        'package_dir'         : {'': 'src'},
        'packages'            : find_packages(str(srcfile('src')), exclude=['tests']),
        'data_files'          : data_files.items(),
        'zip_safe'            : False,
        'author'              : about('author'),
        'author_email'        : about('author_email'),
        'license'             : about('license'),
        'url'                 : about('url'),
        'version'             : about('version'),
        'description'         : about('description'),
        'long_description'    : long_description(),
        'include_package_data': True,
        'setup_requires'      : requires('setup-requirements.txt'),
        'tests_require'       : requires('test-requirements.txt'),
        'install_requires'    : requires('install-requirements.txt'),
        'entry_points'        : entry_points(),
        'use_scm_version'     : True,
        # 'classifiers'         : classifiers, 'entry_points': entry_points()
    }

    return metadata

# Ensure "setup.py" is importable by other tools in order
# to access the project's metadata
project = _build_metadata()
__all__ = ['project', 'project_root', 'package_name', 'srcfile']

if __name__ == '__main__':
    if '--metadata' in sys.argv[:2]:
        json.dump(project, sys.stdout, default=repr, indent=4, sort_keys=True)
        sys.stdout.write('\n')
    else:
        setup(**project)
